/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

import ClayLoadingIndicator from '@clayui/loading-indicator';
import React, {useContext, useEffect, useState} from 'react';

import DataSetContext from '../../../DataSetContext';
import {getComponentByModuleURL} from '../../../utils/modules';
import AutocompleteFilter, {
	getOdataString as getAutocompleteFilterOdataString,
	getSelectedItemsLabel as getAutocompleteFilterSelectedItemsLabel,
} from './AutocompleteFilter';
import CheckboxesFilter, {
	getOdataString as getCheckboxesFilterOdataString,
	getSelectedItemsLabel as getCheckboxesFilterSelectedItemsLabel,
} from './CheckboxesFilter';
import DateRangeFilter, {
	getOdataString as getDateRangeFilterOdataString,
	getSelectedItemsLabel as getDateRangeFilterSelectedItemsLabel,
} from './DateRangeFilter';
import RadioFilter, {
	getOdataString as getRadioFilterOdataString,
	getSelectedItemsLabel as getRadioFilterSelectedItemsLabel,
} from './RadioFilter';

const FILTER_TYPE_COMPONENT = {
	autocomplete: AutocompleteFilter,
	checkbox: CheckboxesFilter,
	dateRange: DateRangeFilter,
	radio: RadioFilter,
};

const getFilterSelectedItemsLabel = (filter) => {
	switch (filter.type) {
		case 'autocomplete':
			return getAutocompleteFilterSelectedItemsLabel(filter);
		case 'checkbox':
			return getCheckboxesFilterSelectedItemsLabel(filter);
		case 'dateRange':
			return getDateRangeFilterSelectedItemsLabel(filter);
		case 'radio':
			return getRadioFilterSelectedItemsLabel(filter);
		default:
			return '';
	}
};

const getOdataFilterString = (filter) => {
	switch (filter.type) {
		case 'autocomplete':
			return getAutocompleteFilterOdataString(filter);
		case 'checkbox':
			return getCheckboxesFilterOdataString(filter);
		case 'dateRange':
			return getDateRangeFilterOdataString(filter);
		case 'radio':
			return getRadioFilterOdataString(filter);
		default:
			return '';
	}
};

const Filter = ({moduleURL, type, ...otherProps}) => {
	const {setFilters} = useContext(DataSetContext);

	const [Component, setComponent] = useState(() => {
		if (!moduleURL) {
			const Matched = FILTER_TYPE_COMPONENT[type];

			if (!Matched) {
				throw new Error(`Filter type '${type}' not found.`);
			}

			return Matched;
		}
		else {
			return null;
		}
	});

	useEffect(() => {
		if (moduleURL) {
			getComponentByModuleURL(moduleURL).then((FetchedComponent) =>
				setComponent(() => FetchedComponent)
			);
		}
	}, [moduleURL]);

	const setFilter = ({id, ...otherProps}) => {
		setFilters((filters) => {
			return filters.map((filter) => ({
				...filter,
				...(filter.id === id ? {...otherProps} : {}),
			}));
		});
	};

	return Component ? (
		<div className="data-set-filter">
			<Component setFilter={setFilter} {...otherProps} />
		</div>
	) : (
		<ClayLoadingIndicator small />
	);
};

export {getFilterSelectedItemsLabel, getOdataFilterString};
export default Filter;
